use std::fmt;

#[derive(Debug, Fail)]
pub enum CoinError {
    #[fail(display = "Not a valid side")]
    InvalidSide,
}

#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq, PartialOrd, Ord)]
pub enum Coin {
    Heads,
    Tails,
}

impl Coin {
    /// Flips a coin, returning a Coin with its current Side up
    pub fn flip() -> Coin {
        Coin::from(rand::random::<bool>())
    }
}

impl fmt::Display for Coin {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(
            f,
            "{}",
            match self {
                Coin::Heads => "Heads",
                Coin::Tails => "Tails",
            }
        )
    }
}

#[derive(Copy, Clone, Debug)]
pub struct CoinToss {
    pub guess: Coin,
    pub real: Coin,
}

impl std::convert::Into<bool> for CoinToss {
    fn into(self) -> bool {
        self.guess == self.real
    }
}

impl std::convert::From<bool> for Coin {
    fn from(b: bool) -> Coin {
        if b {
            Coin::Heads
        } else {
            Coin::Tails
        }
    }
}

impl fmt::Display for CoinToss {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "Guess: {}\nResult: {}", self.guess, self.real)
    }
}

impl CoinToss {
    pub fn guess(coin_side: &str) -> Result<Self, CoinError> {
        if !["heads", "h", "tails", "t"].contains(&coin_side) {
            Err(CoinError::InvalidSide)
        } else {
            Ok(CoinToss {
                guess: Coin::from(coin_side.starts_with('h')),
                real: Coin::flip(),
            })
        }
    }
}
