use crate::cards::StandardCard;
use crate::deck_of_cards::STANDARD_DECK_OF_CARDS;
use rand::{thread_rng, Rng};
use std::error::Error as StdError;
use std::fmt::{Display, Formatter, Result as FmtResult};

#[derive(Clone, Debug, Default)]
pub struct StandardDeck {
    pub cards: Vec<StandardCard>,
}

#[derive(Debug)]
pub enum StandardDeckError {
    NoCard,
}

impl Display for StandardDeckError {
    fn fmt(&self, f: &mut Formatter) -> FmtResult {
        f.write_str(self.description())
    }
}

impl StdError for StandardDeckError {
    fn description(&self) -> &str {
        match *self {
            StandardDeckError::NoCard => "No Cards are left in the deck",
        }
    }
}

impl StandardDeck {
    #[allow(unused_mut)]
    pub fn new<T: Into<Option<usize>>>(sets: T) -> Self {
        let mut rng = thread_rng();
        let mut cards = match sets.into() {
            None => STANDARD_DECK_OF_CARDS.to_vec(),
            Some(sets) => {
                let mut vec = Vec::with_capacity(52 * sets);
                for _ in 0..sets {
                    vec.extend(STANDARD_DECK_OF_CARDS.iter())
                }
                vec
            }
        };

        rng.shuffle(&mut cards);
        Self { cards }
    }

    pub fn draw(&mut self) -> StandardCard {
        // Game should never get to the point where the deck is empty
        // but if it does, create a new deck
        self.cards
            .pop()
            .expect("Game should never run out of cards")
    }
}
