use super::{Weapon, WeaponParseError};
use std::str::FromStr;

/// Result of `RpsGame`
#[derive(Copy, Clone, Debug)]
pub enum GameResult {
    /// Player has won the game
    Win { player: Weapon, robo: Weapon },
    /// Player has lost the game
    Lose { player: Weapon, robo: Weapon },
    /// It is a draw
    Draw { player: Weapon, robo: Weapon },
}

/// Rock Paper scissors Game
#[derive(Copy, Clone, Debug)]
pub struct RpsGame {
    // Player's chosen weapon
    player: Weapon,
    // Bot's chosen Weapon
    robo: Weapon,
}

impl RpsGame {
    pub fn new(r: &str) -> Result<Self, WeaponParseError> {
        Ok(Self {
            player: Weapon::from_str(r)?,
            robo: Weapon::rand(),
        })
    }
    pub fn result(self) -> GameResult {
        self.into()
    }
}

impl std::convert::Into<GameResult> for RpsGame {
    fn into(self) -> GameResult {
        if self.player > self.robo {
            GameResult::Win {
                player: self.player,
                robo: self.robo,
            }
        } else if self.player == self.robo {
            GameResult::Draw {
                player: self.player,
                robo: self.robo,
            }
        } else {
            GameResult::Lose {
                player: self.player,
                robo: self.robo,
            }
        }
    }
}
